@@ -1,4 +1,4 @@
-/* $Id: main.c 28125 2008-10-09 21:46:58Z olivier $ */
+/* $Id: main.c 28560 2008-11-01 15:42:41Z olivier $ */
 /*
  *  Copyright (c) 2008 Stephan Arts <stephan@xfce.org>
  *  Copyright (c) 2008 Jannis Pohlmann <jannis@xfce.org>
@@ -205,6 +205,7 @@ cb_custom_dpi_check_button_toggled (GtkToggleButton *custom_dpi_toggle,
     }
 }
 
+#ifdef ENABLE_SOUND_SETTINGS
 static void
 cb_enable_event_sounds_check_button_toggled (GtkToggleButton *toggle, GtkWidget *button)
 {
@@ -214,6 +215,7 @@ cb_enable_event_sounds_check_button_toggled (GtkToggleButton *toggle, GtkWidget
     gtk_widget_set_sensitive (button, active);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), active);
 }
+#endif
 
 static void
 appearance_settings_load_icon_themes (GtkListStore *list_store,
@@ -234,7 +236,7 @@ appearance_settings_load_icon_themes (GtkListStore *list_store,
     GSList       *check_list = NULL;
 
     /* Determine current theme */
-    active_theme_name = xfconf_channel_get_string (xsettings_channel, "/Net/IconThemeName", "hicolor");
+    active_theme_name = xfconf_channel_get_string (xsettings_channel, "/Net/IconThemeName", "Rodent");
 
     /* Determine directories to look in for icon themes */
     xfce_resource_push_path (XFCE_RESOURCE_ICONS, DATADIR G_DIR_SEPARATOR_S "icons");
@@ -654,10 +656,10 @@ appearance_settings_dialog_configure_widgets (GladeXML *gxml)
     GtkWidget *toolbar_style_combo = glade_xml_get_widget (gxml, "gtk_toolbar_style_combo_box");
 
     list_store = gtk_list_store_new (1, G_TYPE_STRING);
-    gtk_list_store_insert_with_values (list_store, NULL, 0, 0, N_("Icons"), -1);
-    gtk_list_store_insert_with_values (list_store, NULL, 1, 0, N_("Text"), -1);
-    gtk_list_store_insert_with_values (list_store, NULL, 2, 0, N_("Both"), -1);
-    gtk_list_store_insert_with_values (list_store, NULL, 3, 0, N_("Both Horizontal"), -1);
+    gtk_list_store_insert_with_values (list_store, NULL, 0, 0, _("Icons"), -1);
+    gtk_list_store_insert_with_values (list_store, NULL, 1, 0, _("Text"), -1);
+    gtk_list_store_insert_with_values (list_store, NULL, 2, 0, _("Both"), -1);
+    gtk_list_store_insert_with_values (list_store, NULL, 3, 0, _("Both Horizontal"), -1);
 
     renderer = gtk_cell_renderer_text_new ();
     gtk_cell_layout_clear (GTK_CELL_LAYOUT (toolbar_style_combo));
@@ -681,10 +683,10 @@ appearance_settings_dialog_configure_widgets (GladeXML *gxml)
     GtkWidget *hinting_style_combo = glade_xml_get_widget (gxml, "xft_hinting_style_combo_box");
 
     list_store = gtk_list_store_new (1, G_TYPE_STRING);
-    gtk_list_store_insert_with_values (list_store, NULL, 0, 0, N_("None"), -1);
-    gtk_list_store_insert_with_values (list_store, NULL, 1, 0, N_("Slight"), -1);
-    gtk_list_store_insert_with_values (list_store, NULL, 2, 0, N_("Medium"), -1);
-    gtk_list_store_insert_with_values (list_store, NULL, 3, 0, N_("Full"), -1);
+    gtk_list_store_insert_with_values (list_store, NULL, 0, 0, _("None"), -1);
+    gtk_list_store_insert_with_values (list_store, NULL, 1, 0, _("Slight"), -1);
+    gtk_list_store_insert_with_values (list_store, NULL, 2, 0, _("Medium"), -1);
+    gtk_list_store_insert_with_values (list_store, NULL, 3, 0, _("Full"), -1);
 
     renderer = gtk_cell_renderer_text_new ();
     gtk_cell_layout_clear (GTK_CELL_LAYOUT (hinting_style_combo));
@@ -847,9 +849,11 @@ main(gint argc, gchar **argv)
             {
                 /* Create plug widget */
                 plug = gtk_plug_new (opt_socket_id);
+                g_signal_connect (plug, "delete-event", G_CALLBACK (gtk_main_quit), NULL);
                 gtk_widget_show (plug);
 
-                g_signal_connect (plug, "delete-event", G_CALLBACK (gtk_main_quit), NULL);
+                /* Stop startup notification */
+                gdk_notify_startup_complete ();
 
                 /* Get plug child widget */
                 plug_child = glade_xml_get_widget (gxml, "plug-child");
