@@ -40,6 +40,18 @@ static GladeXML *gxml_main_window = NULL;
 static XfconfChannel *current_channel = NULL;
 static gchar *current_property = NULL;
 
+enum {
+    PROP_TYPE_EMPTY,
+    PROP_TYPE_STRING,
+    PROP_TYPE_INT,
+    PROP_TYPE_UINT,
+    PROP_TYPE_INT64,
+    PROP_TYPE_UINT64,
+    PROP_TYPE_DOUBLE,
+    PROP_TYPE_BOOLEAN,
+    PROP_TYPE_ARRAY /* Not used yet */
+};
+
 static void
 load_channels (GtkListStore *store, GtkTreeView *treeview);
 static void
@@ -256,43 +268,41 @@ load_properties (XfconfChannel *channel, GtkTreeStore *store, GtkTreeView *treev
 
                             if (components[i+1] == NULL)
                             {
-                                g_value_init (&property_value, G_TYPE_STRING);
-                                if (!xfconf_channel_get_property (channel, key, &property_value))
-                                {
-                                    g_value_unset (&property_value);
-                                    xfconf_channel_get_property (channel, key, &property_value);
-                                }
+                                xfconf_channel_get_property (channel, key, &property_value);
                                 switch (G_VALUE_TYPE(&property_value))
                                 {
                                     case G_TYPE_INT:
                                         g_value_set_string (&child_type, "Int");
+                                        g_value_transform (&property_value, &child_value);
                                         break;
                                     case G_TYPE_UINT:
                                         g_value_set_string (&child_type, "Unsigned Int");
+                                        g_value_transform (&property_value, &child_value);
                                         break;
                                     case G_TYPE_INT64:
                                         g_value_set_string (&child_type, "Int64");
+                                        g_value_transform (&property_value, &child_value);
                                         break;
                                     case G_TYPE_UINT64:
                                         g_value_set_string (&child_type, "Unsigned Int64");
+                                        g_value_transform (&property_value, &child_value);
                                         break;
                                     case G_TYPE_DOUBLE:
                                         g_value_set_string (&child_type, "Double");
+                                        g_value_transform (&property_value, &child_value);
                                         break;
                                     case G_TYPE_STRING:
                                         g_value_set_string (&child_type, "String");
+                                        g_value_copy (&property_value, &child_value);
                                         break;
                                     case G_TYPE_BOOLEAN:
                                         g_value_set_string (&child_type, "Bool");
+                                        g_value_transform (&property_value, &child_value);
                                         break;
                                     default:
                                         g_value_set_string (&child_type, g_type_name (G_VALUE_TYPE(&property_value)));
                                         break;
                                 }
-                                if (G_VALUE_HOLDS_STRING (&property_value))
-                                {
-                                    g_value_copy (&property_value, &child_value);
-                                }
                                 g_value_unset (&property_value);
                             }
                             else
@@ -320,45 +330,44 @@ load_properties (XfconfChannel *channel, GtkTreeStore *store, GtkTreeView *treev
                     gtk_tree_store_set_value (store, &child_iter, 0, &child_name);
                     g_value_reset (&child_name);
 
-                    if (components[i+1] == NULL) 
+                    if (components[i+1] == NULL)
                     {
-                        g_value_init (&property_value, G_TYPE_STRING);
-                        if (!xfconf_channel_get_property (channel, key, &property_value))
-                        {
-                            g_value_unset (&property_value);
-                            xfconf_channel_get_property (channel, key, &property_value);
-                        }
+                        xfconf_channel_get_property (channel, key, &property_value);
                         switch (G_VALUE_TYPE(&property_value))
                         {
                             case G_TYPE_INT:
                                 g_value_set_string (&child_type, "Int");
+                                g_value_transform (&property_value, &child_value);
                                 break;
                             case G_TYPE_UINT:
                                 g_value_set_string (&child_type, "Unsigned Int");
+                                g_value_transform (&property_value, &child_value);
                                 break;
                             case G_TYPE_INT64:
                                 g_value_set_string (&child_type, "Int64");
+                                g_value_transform (&property_value, &child_value);
                                 break;
                             case G_TYPE_UINT64:
                                 g_value_set_string (&child_type, "Unsigned Int64");
+                                g_value_transform (&property_value, &child_value);
                                 break;
                             case G_TYPE_DOUBLE:
                                 g_value_set_string (&child_type, "Double");
+                                g_value_transform (&property_value, &child_value);
                                 break;
                             case G_TYPE_STRING:
                                 g_value_set_string (&child_type, "String");
+                                g_value_copy (&property_value, &child_value);
                                 break;
                             case G_TYPE_BOOLEAN:
                                 g_value_set_string (&child_type, "Bool");
+                                g_value_transform (&property_value, &child_value);
                                 break;
                             default:
                                 g_value_set_string (&child_type, g_type_name (G_VALUE_TYPE(&property_value)));
+                                g_value_set_string (&child_value, "...");
                                 break;
                         }
-                        if (G_VALUE_HOLDS_STRING (&property_value))
-                        {
-                            g_value_copy (&property_value, &child_value);
-                        }
                         g_value_unset (&property_value);
                     }
                     else
@@ -371,6 +380,9 @@ load_properties (XfconfChannel *channel, GtkTreeStore *store, GtkTreeView *treev
                     g_value_set_boolean (&child_locked, xfconf_channel_is_property_locked (channel, key));
                     gtk_tree_store_set_value (store, &child_iter, 2, &child_locked);
                     g_value_reset (&child_locked);
+
+                    gtk_tree_store_set_value (store, &child_iter, 3, &child_value);
+                    g_value_reset (&child_value);
                 }
                 parent_iter = child_iter;
             }
@@ -485,27 +497,137 @@ cb_property_new_button_clicked (GtkButton *button, gpointer user_data)
 static void
 cb_property_edit_button_clicked (GtkButton *button, gpointer user_data)
 {
-    GtkTreeModel *model;
     GValue value = {0, };
     gchar *prop_name = NULL;
-    gchar *temp = NULL;
-    GtkWidget *channel_treeview = glade_xml_get_widget (gxml_main_window, "channel_treeview");
 
+    GtkWidget *property_treeview = glade_xml_get_widget (gxml_main_window, "property_treeview");
+    GtkTreeModel *tree_store = gtk_tree_view_get_model (GTK_TREE_VIEW (property_treeview));
     GtkWidget *dialog = glade_xml_get_widget (gxml_main_window, "edit_settings_dialog");
     GtkWidget *prop_name_entry = glade_xml_get_widget (gxml_main_window, "property_name_entry");
+    GtkWidget *prop_type_combo = glade_xml_get_widget (gxml_main_window, "property_type_combo");
+
+    GtkWidget *prop_value_text_entry = glade_xml_get_widget (gxml_main_window, "property_value_text_entry");
+    GtkWidget *prop_value_spin_button = glade_xml_get_widget (gxml_main_window, "property_value_spin_button");
+    GtkWidget *prop_value_sw = glade_xml_get_widget (gxml_main_window, "property_value_sw");
+    GtkWidget *prop_value_checkbox = glade_xml_get_widget (gxml_main_window, "property_value_checkbutton");
 
 
     /* Set the correct properties in the ui */
     gtk_entry_set_text (GTK_ENTRY(prop_name_entry), current_property);
     if (xfconf_channel_get_property (current_channel, current_property, &value))
     {
-        
+        switch (G_VALUE_TYPE(&value))      
+        {
+            case G_TYPE_STRING:
+                gtk_combo_box_set_active (GTK_COMBO_BOX (prop_type_combo), PROP_TYPE_STRING);
+                gtk_widget_hide (prop_value_spin_button);
+                gtk_widget_show (prop_value_text_entry);
+                gtk_widget_hide (prop_value_sw);
+                gtk_widget_hide (prop_value_checkbox);
+                gtk_entry_set_text (GTK_ENTRY (prop_value_text_entry), g_value_get_string (&value));
+                break;
+            case G_TYPE_INT:
+                gtk_combo_box_set_active (GTK_COMBO_BOX (prop_type_combo), PROP_TYPE_INT);
+                gtk_widget_show (prop_value_spin_button);
+                gtk_widget_hide (prop_value_text_entry);
+                gtk_widget_hide (prop_value_sw);
+                gtk_widget_hide (prop_value_checkbox);
+                gtk_spin_button_set_value (GTK_SPIN_BUTTON (prop_value_spin_button), g_value_get_int (&value));
+                gtk_spin_button_set_range (GTK_SPIN_BUTTON (prop_value_spin_button), G_MININT, G_MAXINT);
+                gtk_spin_button_set_digits (GTK_SPIN_BUTTON (prop_value_spin_button), 0);
+                break;
+            case G_TYPE_UINT:
+                gtk_combo_box_set_active (GTK_COMBO_BOX (prop_type_combo), PROP_TYPE_UINT);
+                gtk_widget_show (prop_value_spin_button);
+                gtk_widget_hide (prop_value_text_entry);
+                gtk_widget_hide (prop_value_sw);
+                gtk_widget_hide (prop_value_checkbox);
+                gtk_spin_button_set_value (GTK_SPIN_BUTTON (prop_value_spin_button), g_value_get_uint (&value));
+                gtk_spin_button_set_range (GTK_SPIN_BUTTON (prop_value_spin_button), 0, G_MAXINT);
+                gtk_spin_button_set_digits (GTK_SPIN_BUTTON (prop_value_spin_button), 0);
+                break;
+            case G_TYPE_INT64:
+                gtk_combo_box_set_active (GTK_COMBO_BOX (prop_type_combo), PROP_TYPE_INT64);
+                gtk_widget_show (prop_value_spin_button);
+                gtk_widget_hide (prop_value_text_entry);
+                gtk_widget_hide (prop_value_sw);
+                gtk_widget_hide (prop_value_checkbox);
+                gtk_spin_button_set_value (GTK_SPIN_BUTTON (prop_value_spin_button), g_value_get_int64 (&value));
+                gtk_spin_button_set_range (GTK_SPIN_BUTTON (prop_value_spin_button), G_MININT64, G_MAXINT64);
+                gtk_spin_button_set_digits (GTK_SPIN_BUTTON (prop_value_spin_button), 0);
+                break;
+            case G_TYPE_UINT64:
+                gtk_combo_box_set_active (GTK_COMBO_BOX (prop_type_combo), PROP_TYPE_UINT64);
+                gtk_widget_show (prop_value_spin_button);
+                gtk_widget_hide (prop_value_text_entry);
+                gtk_widget_hide (prop_value_sw);
+                gtk_widget_hide (prop_value_checkbox);
+                gtk_spin_button_set_value (GTK_SPIN_BUTTON (prop_value_spin_button), g_value_get_uint64 (&value));
+                gtk_spin_button_set_range (GTK_SPIN_BUTTON (prop_value_spin_button), 0, G_MAXUINT64);
+                gtk_spin_button_set_digits (GTK_SPIN_BUTTON (prop_value_spin_button), 0);
+                break;
+            case G_TYPE_DOUBLE:
+                gtk_combo_box_set_active (GTK_COMBO_BOX (prop_type_combo), PROP_TYPE_DOUBLE);
+                gtk_widget_show (prop_value_spin_button);
+                gtk_widget_hide (prop_value_text_entry);
+                gtk_widget_hide (prop_value_sw);
+                gtk_widget_hide (prop_value_checkbox);
+                gtk_spin_button_set_value (GTK_SPIN_BUTTON (prop_value_spin_button), g_value_get_double (&value));
+                gtk_spin_button_set_range (GTK_SPIN_BUTTON (prop_value_spin_button), G_MINDOUBLE, G_MAXDOUBLE);
+                gtk_spin_button_set_digits (GTK_SPIN_BUTTON (prop_value_spin_button), 2);
+                break;
+            case G_TYPE_BOOLEAN:
+                gtk_combo_box_set_active (GTK_COMBO_BOX (prop_type_combo), PROP_TYPE_BOOLEAN);
+                gtk_widget_hide (prop_value_spin_button);
+                gtk_widget_hide (prop_value_text_entry);
+                gtk_widget_hide (prop_value_sw);
+                gtk_widget_show (prop_value_checkbox);
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prop_value_checkbox), g_value_get_boolean (&value));
+                break;
+            default:
+                return;
+                break;
+        }
+    }
+    else
+    {
+        gtk_combo_box_set_active (GTK_COMBO_BOX (prop_type_combo), PROP_TYPE_EMPTY);
     }
     
     if (gtk_dialog_run (GTK_DIALOG(dialog)) == GTK_RESPONSE_APPLY)
     {
-        
         gtk_widget_hide (dialog);
+        switch (gtk_combo_box_get_active (GTK_COMBO_BOX (prop_type_combo)))
+        {
+            case PROP_TYPE_EMPTY:
+                break;
+            case PROP_TYPE_STRING:
+                g_value_set_string (&value, gtk_entry_get_text (GTK_ENTRY (prop_value_text_entry))); 
+                break;
+            case PROP_TYPE_INT:
+                g_value_set_int (&value, gtk_spin_button_get_value (GTK_SPIN_BUTTON (prop_value_spin_button))); 
+                break;
+            case PROP_TYPE_UINT:
+                g_value_set_uint (&value, gtk_spin_button_get_value (GTK_SPIN_BUTTON (prop_value_spin_button))); 
+                break;
+            case PROP_TYPE_INT64:
+                g_value_set_int64 (&value, gtk_spin_button_get_value (GTK_SPIN_BUTTON (prop_value_spin_button))); 
+                break;
+            case PROP_TYPE_UINT64:
+                g_value_set_uint64 (&value, gtk_spin_button_get_value (GTK_SPIN_BUTTON (prop_value_spin_button))); 
+                break;
+            case PROP_TYPE_DOUBLE:
+                g_value_set_double (&value, gtk_spin_button_get_value (GTK_SPIN_BUTTON (prop_value_spin_button))); 
+                break;
+            case PROP_TYPE_BOOLEAN:
+                g_value_set_boolean (&value, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (prop_value_checkbox))); 
+                break;
+            case PROP_TYPE_ARRAY:
+                break;
+        }   
+        xfconf_channel_set_property (current_channel, current_property, &value);
+        gtk_tree_store_clear (GTK_TREE_STORE(tree_store));
+        load_properties (current_channel, GTK_TREE_STORE (tree_store), GTK_TREE_VIEW (property_treeview));
     }
     else
     {
@@ -516,6 +638,11 @@ cb_property_edit_button_clicked (GtkButton *button, gpointer user_data)
         g_free (prop_name);
 }
 
+/**
+ * cb_property_revert_button_clicked
+ *
+ * Resets a property to it's system-default, it removes the property if it does not exist as a system default.
+ */
 static void
 cb_property_revert_button_clicked (GtkButton *button, gpointer user_data)
 {
@@ -538,7 +665,7 @@ cb_property_revert_button_clicked (GtkButton *button, gpointer user_data)
         dialog = gtk_message_dialog_new_with_markup (
                                      GTK_WINDOW (glade_xml_get_widget (gxml_main_window, "main_window")),
                                      0, GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO,
-                                     "Are you sure you wat to reset property:\n<b>%s</b>?",
+                                     "Are you sure you wat to reset this property:\n<b>%s</b>?",
                                      current_property);
         if (gtk_dialog_run (GTK_DIALOG(dialog)) == GTK_RESPONSE_YES)
         {
@@ -546,6 +673,7 @@ cb_property_revert_button_clicked (GtkButton *button, gpointer user_data)
             tree_store = gtk_tree_view_get_model (GTK_TREE_VIEW (property_treeview));
             gtk_widget_hide (dialog);
             xfconf_channel_reset_property (current_channel, current_property, FALSE);
+            gtk_tree_store_clear (GTK_TREE_STORE(tree_store));
             load_properties (current_channel, GTK_TREE_STORE (tree_store), GTK_TREE_VIEW (property_treeview));
         }
         gtk_widget_destroy (dialog);
